-- HUM, main.lua
-- version 7
-- EDITED FROM ORIGINAL by Pizzasgood
--     * Use (++) instead of (+), and so on
--     * Optionally show ship
--     * Optionally don't hijack GetFriendlyStatus (for when you temporarily want the normal functionality without giving up the notifications)
--     * Save settings

HUM = {}
HUM.Players = {}

HUM.maincolor = "\127FFFFFF"
HUM.subcolor = "\127B7B7B7"
HUM.truecolor = "\12700FF00"
HUM.falsecolor = "\127FF0000"

--Lua doesn't treat 0 as false, and gkini doesn't come with a WriteBool/ReadBool, so let's just roll our own.
local function ReadBool(namespace, attribute, default)
        local d
        if default then d = 1 else d = 0 end
        return ( gkini.ReadInt(namespace, attribute, d) ~= 0 )
end
local function WriteBool(namespace, attribute, value)
        local v
        if value then v = 1 else v = 0 end
        gkini.WriteInt(namespace, attribute, v)
end

--Print booleans in color
local function colorprint(val)
        if val then
                return(HUM.truecolor.."On"..'\127o')
        else
                return(HUM.falsecolor.."Off"..'\127o')
        end
end

--Load settings
HUM.On = ReadBool('HUM', 'active', true)
HUM.RadarOn = ReadBool('HUM', 'radar', true)
HUM.ShipOn = ReadBool('HUM', 'ship', true)

function HUM:toggle (_)
        if HUM.On then
                HUM.On = false
                GetFriendlyStatus = _GetFriendlyStatus
                HUM.KillScanner()
        else
                HUM.On = true
                GetFriendlyStatus = HUM.GetFriendlyStatus
                HUM.Scanner()
        end
        print(HUM.maincolor .. 'HUM:\127o ' .. colorprint(HUM.On))
        WriteBool('HUM', 'active', HUM.On)
end

function HUM:toggleRadar (_)
        if HUM.RadarOn then
                HUM.RadarOn = false
                print(HUM.maincolor .. 'HUM radar:\127o ' .. HUM.falsecolor .. 'Normal radar\127o' )
        else
                HUM.RadarOn = true
                print(HUM.maincolor .. 'HUM radar:\127o ' .. HUM.truecolor .. 'Proximity radar\127o' )
        end
        WriteBool('HUM', 'radar', HUM.RadarOn)
end

function HUM:toggleShip (_)
        if HUM.ShipOn then
                HUM.ShipOn = false
        else
                HUM.ShipOn = true
        end
        print(HUM.maincolor .. 'HUM ship:\127o ' .. colorprint(HUM.ShipOn))
        WriteBool('HUM', 'ship', HUM.ShipOn)
end

function HUM:usage (_)
        print(HUM.maincolor .. 'HUM usage:\127o')
        print(HUM.maincolor .. 'HUM\127o ' .. HUM.subcolor .. 'toggle plugin\127o')
        print(HUM.maincolor .. 'HUM radar\127o ' .. HUM.subcolor .. 'toggle whether radar shows proximity or hostility\127o')
        print(HUM.maincolor .. 'HUM ship\127o ' .. HUM.subcolor .. 'toggle whether ship type is included in notice\127o')
        print(HUM.maincolor .. 'HUM print\127o ' .. HUM.subcolor .. 'print current status\127o')
end

function HUM:status (_)
        print(HUM.maincolor .. 'HUM:\127o ' .. colorprint(HUM.On) .. '\127o')
        print(HUM.maincolor .. 'HUM radar:\127o ' .. colorprint(HUM.RadarOn) .. '\127o')
        print(HUM.maincolor .. 'HUM ship:\127o ' .. colorprint(HUM.ShipOn) .. '\127o')
end

function HUM.GetFriendlyStatus (charid)
        if HUM.RadarOn then
                if HUM.Players[charid] and HUM.Players[charid].proximate then
                        return 0
                elseif charid == 0 then
                        return 0
                else
                        return 3
                end
        else
                return _GetFriendlyStatus(charid)
        end
end

_GetFriendlyStatus = GetFriendlyStatus
if HUM.On then
        GetFriendlyStatus = HUM.GetFriendlyStatus
end

function HUM:OnEvent (event, args)
        if event == 'PROXIMITY_ALERT' or event == 'PLAYER_ENTERED_GAME' then
                HUM.Scanner()
        elseif args ~= 0 and args ~= GetCharacterID() then
                if HUM.GetName(args) and not (event ~= 'PLAYER_LEFT_SECTOR' and HUM.Players[args].announced) then
                        HUM.Alert(event, args)
                end
        end
end

function HUM.KillScanner()
        if HUM.timer and HUM.timer:IsActive() then
                HUM.timer:Kill()
        end
end
       

function HUM.Scanner()
        HUM.KillScanner()
        HUM.timer = Timer()
        HUM.Scan()
        HUM.timer:SetTimeout(500, function () HUM.Scanner() end)
end

function HUM.Scan()
        ForEachPlayer(function (charid) if charid ~= 0 and charid ~= GetCharacterID() and HUM.GetName(charid) then 
                                                if not HUM.Players[charid].announced then
                                                        HUM.Alert('PLAYER_ENTERED_SECTOR', charid)
                                                end
                                                local d = GetPlayerDistance(charid)
                                                if d and d ~= 0 and d <= GetProximityWarningDistance() then
                                                        if not HUM.Players[charid].proximate then
                                                                HUM.Players[charid].proximate = true
                                                                if not IsGroupMember(charid) and not IsGuildMember(charid) then
                                                                        HUM.Alert("PROXIMITY_ALERT", charid)
                                                                end
                                                        end
                                                elseif HUM.Players[charid] then
                                                        HUM.Players[charid].proximate = false
                                        end end end)
end


function HUM.Alert (e, p)
        local function header ()
                local function colorize (s, sc)
                        local c = HUM.subcolor
                        local e = '\127o'
                        return c .. '(' .. e .. sc .. s .. e .. c .. ') ' .. e
                end
                if e == 'PLAYER_LEFT_SECTOR' then
                        HUM.Players[p].announced = false
                        return colorize('--', '\12700FF00')
                elseif e == 'PROXIMITY_ALERT' then
                        return colorize('!!', '\127FF0000')
                else
                        HUM.Players[p].announced = true
                        return colorize('++', '\127FFFF00')
                end
        end
        local ship = ""
        if HUM.ShipOn and GetPrimaryShipNameOfPlayer(p) ~= nil then
                        ship = HUM.subcolor .. " in a \127o" .. HUM.maincolor .. GetPrimaryShipNameOfPlayer(p) .. "\127o"
                end
        if HUM.On and string.sub(HUM.Players[p].name, 1, 1) ~= "*" then
                print(header() .. rgbtohex(FactionColor_RGB[HUM.Players[p].faction]) .. HUM.Players[p].name .. "\127o" .. ship )
        end
end


function HUM.GetName (p)
        if HUM.Players[p] and HUM.Players[p].name then
                return true
        else
                local n = GetPlayerName(p)
                local f = GetPlayerFaction(p)
                if ( not f ) or n == "(reading transponder " .. p .. ")" then
                        return false
                else
                        HUM.Players[p] = {}
                        HUM.Players[p].name = n
                        HUM.Players[p].faction = f
                        HUM.Players[p].proximate = false
                        return true
                end
        end
end

RegisterEvent(HUM, 'PLAYER_ENTERED_GAME')
RegisterEvent(HUM, 'PLAYER_ENTERED_SECTOR')
RegisterEvent(HUM, 'PLAYER_LEFT_SECTOR')
RegisterEvent(HUM, 'PROXIMITY_ALERT')
RegisterEvent(HUM, 'UPDATE_CHARINFO')
RegisterUserCommand("hum", function(_,data)
                                        if data == nil then
                                                HUM.toggle()
                                        elseif data[1] == "print" or data[1] == "p" then
                                                HUM.status()
                                        elseif data[1] == "radar" or data[1] == "r" then
                                                HUM.toggleRadar()
                                        elseif data[1] == "ship" or data[1] == "s" then
                                                HUM.toggleShip()
                                        else
                                                HUM.usage()
                                        end
                                end)
